==> cl_appbase-0.0.9.tar.gz: lib/src/ioc/service_host.dart <==
import 'package:boost/boost.dart';
import 'service.dart';

/// Convenience function for registering a service on the [ServiceHost]
/// singleton instance.
///
/// See [ServiceHost.register]
void register<T extends Service>(T service) =>
    ServiceHost().register<T>(service);

/// Convenience function for resolving a service in the [ServiceHost]
/// singleton instance.
///
/// See [ServiceHost.get]
T resolve<T extends Service>() => ServiceHost().get<T>();

/// IoC container
///
/// Register a service:
/// ```dart
/// ServiceHost().register(MyServiceImpl());
/// ```
///
/// Receive a service:
/// ```
/// final myService = ServiceHost().get<MyServiceImpl>();
/// ```
/// After registering one or more services, [initializeServices] must be called
/// before receiving one of the newly registered service instances.
///
/// Interfaces for services can be used to enable switching out
/// [Service] implementations more easily.
///
/// Register a service with interface:
/// ```dart
/// ServiceHost().register<MyService>(MyServiceImpl());
/// ```
///
/// Receiving a service via interface class:
/// ```dart
/// // returns the MyServiceImpl from above example
/// final myService = ServiceHost().get<MyService>();
/// ```
///
/// Services and interfaces must implement or extend abstract class [Service].
class ServiceHost {
  static ServiceHost? _instance;

  ServiceHost._();

  factory ServiceHost() => _instance ??= ServiceHost._();

  // Tuple.a -> implementation
  // Tuple.b -> initialized?
  final _services = <Type, Tuple<Service, bool>>{};

  /// Registers a service implementation for type [T].
  ///
  /// If a service for type [T] already exists, this throws an exception.
  /// To replace a service, call and await [dispose] first to make sure the
  /// service is properly disposed.
  void register<T extends Service>(T impl) {
    if (_services.containsKey(T)) {
      throw Exception('Service of type $T already registered.');
    }

    _services[T] = Tuple(impl, false);
  }

  /// Returns true if a service of type [T] is registered.
  bool contains<T>() {
    return _services.containsKey(T);
  }

  /// Returns the registered service implementation for type [T].
  T get<T extends Service>() {
    if (_services.containsKey(T)) {
      final tuple = _services[T]!;
      if (tuple.b) {
        return tuple.a as T;
      } else {
        throw Exception('''Service $T not initialized, call 
            ServiceHost.initializeServices() after registration!''');
      }
    } else {
      throw Exception('Service of type $T not registered.');
    }
  }

  /// Initializes all un-initialized registered services.
  ///
  /// After registering one or more services, this method must be called
  /// to initialize them, or [get] will throw an exception.
  Future<void> initializeServices() async {
    for (final service in _services.values.where((e) => !e.b).map((e) => e.a)) {
      await service.initialize();
    }
  }

  /// Disposes and removes the service registered for type [T] from the host.
  Future<void> dispose<T>() async {
    if (_services.containsKey(T)) {
      final tuple = _services[T]!;
      _services.remove(tuple);
      if (tuple.b) {
        await tuple.a.dispose();
      }
    } else {
      throw Exception('Service of type $T not registered.');
    }
  }
}

==> cl_appbase-0.0.46.tar.gz: lib/src/ioc/service_host.dart <==
import 'package:boost/boost.dart';
import 'service.dart';

/// Convenience function for registering a service on the [ServiceHost]
/// singleton instance.
///
/// See [ServiceHost.register]
void register<T extends Service>(T service) =>
    ServiceHost().register<T>(service);

/// Convenience function for resolving a service in the [ServiceHost]
/// singleton instance.
///
/// See [ServiceHost.get]
T resolve<T extends Service>() => ServiceHost().get<T>();

/// IoC container
///
/// Register a service:
/// ```dart
/// ServiceHost().register(MyServiceImpl());
/// ```
///
/// Receive a service:
/// ```
/// final myService = ServiceHost().get<MyServiceImpl>();
/// ```
/// After registering one or more services, [initializeServices] must be called
/// before receiving one of the newly registered service instances.
///
/// Interfaces for services can be used to enable switching out
/// [Service] implementations more easily.
///
/// Register a service with interface:
/// ```dart
/// ServiceHost().register<MyService>(MyServiceImpl());
/// ```
///
/// Receiving a service via interface class:
/// ```dart
/// // returns the MyServiceImpl from above example
/// final myService = ServiceHost().get<MyService>();
/// ```
///
/// Services and interfaces must implement or extend abstract class [Service].
class ServiceHost {
  static ServiceHost? _instance;

  ServiceHost._();

  factory ServiceHost() => _instance ??= ServiceHost._();

  // Tuple.a -> implementation
  // Tuple.b -> initialized?
  final _services = <Type, Tuple<Service, bool>>{};

  /// Registers a service implementation for type [T].
  ///
  /// If a service for type [T] already exists, this throws an exception.
  /// To replace a service, call and await [dispose] first to make sure the
  /// service is properly disposed.
  void register<T extends Service>(T impl) {
    if (_services.containsKey(T)) {
      throw Exception('Service of type $T already registered.');
    }

    _services[T] = Tuple(impl, false);
  }

  /// Returns true if a service of type [T] is registered.
  bool contains<T>() {
    return _services.containsKey(T);
  }

  /// Returns the registered service implementation for type [T].
  T get<T extends Service>() {
    if (_services.containsKey(T)) {
      final tuple = _services[T]!;
      if (tuple.b) {
        return tuple.a as T;
      } else {
        throw Exception('''Service $T not initialized, call 
            ServiceHost.initializeServices() after registration!''');
      }
    } else {
      throw Exception('Service of type $T not registered.');
    }
  }

  /// Initializes all un-initialized registered services.
  ///
  /// After registering one or more services, this method must be called
  /// to initialize them, or [get] will throw an exception.
  Future<void> initializeServices() async {
    for (final serviceEntry
        in _services.entries.where((e) => !e.value.b).toList()) {
      await serviceEntry.value.a.initialize();
      _services[serviceEntry.key] = serviceEntry.value.withB(true);
    }
  }

  /// Disposes and removes the service registered for type [T] from the host.
  Future<void> dispose<T>() async {
    if (_services.containsKey(T)) {
      final tuple = _services[T]!;
      _services.remove(tuple);
      if (tuple.b) {
        await tuple.a.dispose();
      }
    } else {
      throw Exception('Service of type $T not registered.');
    }
  }
}
